@@ -2,34 +2,30 @@ package Mojolicious::Controller;
 use Mojo::Base -base;
 
 # No imports, for security reasons!
-use Carp ();
+use Carp        ();
+use Digest::SHA ();
 use Mojo::ByteStream;
+use Mojo::DynamicMethods -dispatch;
 use Mojo::URL;
 use Mojo::Util;
 use Mojolicious::Routes::Match;
 use Scalar::Util ();
 
-has [qw(app tx)];
-has match =>
-  sub { Mojolicious::Routes::Match->new(root => shift->app->routes) };
+my $ABSOLUTE = qr!^(?:[^:/?#]+:|//|#)!;
 
-# Reserved stash values
-my %RESERVED = map { $_ => 1 } (
-  qw(action app cb controller data extends format handler inline json layout),
-  qw(namespace path status template text variant)
-);
+has [qw(app tx)] => undef, weak => 1;
+has match => sub { Mojolicious::Routes::Match->new(root => shift->app->routes) };
 
-sub AUTOLOAD {
-  my $self = shift;
+sub BUILD_DYNAMIC {
+  my ($class, $method, $dyn_methods) = @_;
 
-  my ($package, $method) = our $AUTOLOAD =~ /^(.+)::(.+)$/;
-  Carp::croak "Undefined subroutine &${package}::$method called"
-    unless Scalar::Util::blessed $self && $self->isa(__PACKAGE__);
-
-  # Call helper with current controller
-  Carp::croak qq{Can't locate object method "$method" via package "$package"}
-    unless my $helper = $self->app->renderer->get_helper($method);
-  return $self->$helper(@_);
+  return sub {
+    my $self    = shift;
+    my $dynamic = $dyn_methods->{$self->{app}{renderer}}{$method};
+    return $self->$dynamic(@_) if $dynamic;
+    my $package = ref $self;
+    Carp::croak qq{Can't locate object method "$method" via package "$package"};
+  };
 }
 
 sub continue { $_[0]->app->routes->continue($_[0]) }
@@ -41,9 +37,8 @@ sub cookie {
   if (@_) {
 
     # Cookie too big
-    my $cookie = {name => $name, value => shift, %{shift || {}}};
-    $self->app->log->error(qq{Cookie "$name" is bigger than 4096 bytes})
-      if length $cookie->{value} > 4096;
+    my $cookie = {name => $name, value => shift, %{shift // {}}};
+    $self->helpers->log->error(qq{Cookie "$name" is bigger than 4KiB}) if length $cookie->{value} > 4096;
 
     $self->res->cookies($cookie);
     return $self;
@@ -54,16 +49,14 @@ sub cookie {
   return $cookie->value;
 }
 
-sub every_cookie {
-  [map { $_->value } @{shift->req->every_cookie(shift)}];
-}
+sub every_cookie { [map { $_->value } @{shift->req->every_cookie(shift)}] }
 
 sub every_param {
   my ($self, $name) = @_;
 
   # Captured unreserved values
-  my $captures = $self->stash->{'mojo.captures'} ||= {};
-  if (!$RESERVED{$name} && exists $captures->{$name}) {
+  my $captures = $self->stash->{'mojo.captures'} //= {};
+  if (exists $captures->{$name} && !$self->app->routes->is_reserved($name)) {
     my $value = $captures->{$name};
     return ref $value eq 'ARRAY' ? $value : [$value];
   }
@@ -87,15 +80,15 @@ sub every_signed_cookie {
 
       my $valid;
       for my $secret (@$secrets) {
-        my $check = Mojo::Util::hmac_sha1_sum($value, $secret);
+        my $check = Digest::SHA::hmac_sha256_hex("$name=$value", $secret);
         ++$valid and last if Mojo::Util::secure_compare($signature, $check);
       }
       if ($valid) { push @results, $value }
 
-      else { $self->app->log->debug(qq{Cookie "$name" has a bad signature}) }
+      else { $self->helpers->log->trace(qq{Cookie "$name" has bad signature}) }
     }
 
-    else { $self->app->log->debug(qq{Cookie "$name" is not signed}) }
+    else { $self->helpers->log->trace(qq{Cookie "$name" is not signed}) }
   }
 
   return \@results;
@@ -105,38 +98,21 @@ sub finish {
   my $self = shift;
 
   # WebSocket
-  my $tx = $self->tx || Carp::croak 'Connection already closed';
-  $tx->finish(@_) and return $tx->established ? $self : $self->rendered(101)
-    if $tx->is_websocket;
+  my $tx = $self->tx || Carp::croak 'Transaction already destroyed';
+  $tx->finish(@_) and return $tx->established ? $self : $self->rendered(101) if $tx->is_websocket;
 
   # Chunked stream
-  return @_ ? $self->write_chunk(@_)->write_chunk('') : $self->write_chunk('')
-    if $tx->res->content->is_chunked;
+  return @_ ? $self->write_chunk(@_)->write_chunk('') : $self->write_chunk('') if $tx->res->content->is_chunked;
 
   # Normal stream
   return @_ ? $self->write(@_)->write('') : $self->write('');
 }
 
-sub flash {
-  my $self = shift;
-
-  # Check old flash
-  my $session = $self->session;
-  return $session->{flash} ? $session->{flash}{$_[0]} : undef
-    if @_ == 1 && !ref $_[0];
-
-  # Initialize new flash and merge values
-  my $values = ref $_[0] ? $_[0] : {@_};
-  @{$session->{new_flash} ||= {}}{keys %$values} = values %$values;
-
-  return $self;
-}
-
 sub helpers { $_[0]->app->renderer->get_helper('')->($_[0]) }
 
 sub on {
   my ($self, $name, $cb) = @_;
-  my $tx = $self->tx || Carp::croak 'Connection already closed';
+  my $tx = $self->tx || Carp::croak 'Transaction already destroyed';
   $self->rendered(101) if $tx->is_websocket && !$tx->established;
   return $tx->on($name => sub { shift; $self->$cb(@_) });
 }
@@ -148,15 +124,6 @@ sub param {
   return $self;
 }
 
-sub redirect_to {
-  my $self = shift;
-
-  # Don't override 3xx status
-  my $res = $self->res;
-  $res->headers->location($self->url_for(@_));
-  return $self->rendered($res->is_redirect ? () : 302);
-}
-
 sub render {
   my $self = shift;
 
@@ -165,21 +132,28 @@ sub render {
   $args->{template} = $template if $template;
   my $app     = $self->app;
   my $plugins = $app->plugins->emit_hook(before_render => $self, $args);
-  my $maybe   = delete $args->{'mojo.maybe'};
 
-  my $ts = $args->{'mojo.string'};
-  my ($output, $format) = $app->renderer->render($self, $args);
+  # Localize "extends" and "layout" to allow argument overrides
+  my ($maybe, $ts) = @{$args}{'mojo.maybe', 'mojo.string'};
+  my $stash = $self->stash;
+  local $stash->{layout}  = $stash->{layout}  if exists $stash->{layout};
+  local $stash->{extends} = $stash->{extends} if exists $stash->{extends};
+
+  # Rendering to string
+  local @{$stash}{keys %$args}         if $ts || $maybe;
+  delete @{$stash}{qw(layout extends)} if $ts;
+
+  # All other arguments just become part of the stash
+  @$stash{keys %$args} = values %$args;
+  my $renderer = $app->renderer;
+  my ($output, $format) = $renderer->render($self);
 
   # Maybe no 404
   return defined $output ? Mojo::ByteStream->new($output) : undef if $ts;
-  return $maybe ? undef : !$self->helpers->reply->not_found
-    unless defined $output;
+  return $maybe          ? undef : Carp::croak("Could not render a response") unless defined $output;
 
   $plugins->emit_hook(after_render => $self, \$output, $format);
-  my $headers = $self->res->body($output)->headers;
-  $headers->content_type($app->types->type($format) || 'text/plain')
-    unless $headers->content_type;
-  return !!$self->rendered($self->stash->{status});
+  return $renderer->respond($self, $output, $format, $stash->{status});
 }
 
 sub render_later { shift->stash('mojo.rendered' => 1) }
@@ -192,22 +166,24 @@ sub rendered {
   my ($self, $status) = @_;
 
   # Make sure we have a status
-  my $res = $self->res;
-  $res->code($status || 200) if $status || !$res->code;
+  $self->res->code($status) if $status;
 
   # Finish transaction
   my $stash = $self->stash;
   if (!$stash->{'mojo.finished'} && ++$stash->{'mojo.finished'}) {
+    my $res = $self->res;
+    $res->code(200) if !$status && !$res->code;
 
     # Disable auto rendering and stop timer
-    my $app    = $self->render_later->app;
-    my $timing = $self->helpers->timing;
-    if (defined(my $elapsed = $timing->elapsed('mojo.timer'))) {
-      my $rps  = $timing->rps($elapsed) // '??';
-      my $code = $res->code;
-      my $msg  = $res->message || $res->default_message($code);
-      $app->log->debug("$code $msg (${elapsed}s, $rps/s)");
-    }
+    my $app = $self->render_later->app;
+    $self->helpers->log->trace(sub {
+      my $timing  = $self->helpers->timing;
+      my $elapsed = $timing->elapsed('mojo.timer') // 0;
+      my $rps     = $timing->rps($elapsed)         // '??';
+      my $code    = $res->code;
+      my $msg     = $res->message || $res->default_message($code);
+      return "$code $msg (${elapsed}s, $rps/s)";
+    }) unless $stash->{'mojo.static'};
 
     $app->plugins->emit_hook_reverse(after_dispatch => $self);
     $app->sessions->store($self);
@@ -216,39 +192,13 @@ sub rendered {
   return $self;
 }
 
-sub req { (shift->tx || Carp::croak 'Connection already closed')->req }
-sub res { (shift->tx || Carp::croak 'Connection already closed')->res }
-
-sub respond_to {
-  my ($self, $args) = (shift, ref $_[0] ? $_[0] : {@_});
-
-  # Find target
-  my $target;
-  my $renderer = $self->app->renderer;
-  my @formats  = @{$renderer->accepts($self)};
-  for my $format (@formats ? @formats : ($renderer->default_format)) {
-    next unless $target = $args->{$format};
-    $self->stash->{format} = $format;
-    last;
-  }
-
-  # Fallback
-  unless ($target) {
-    return $self->rendered(204) unless $target = $args->{any};
-    delete $self->stash->{format};
-  }
-
-  # Dispatch
-  ref $target eq 'CODE' ? $target->($self) : $self->render(%$target);
-
-  return $self;
-}
+sub req { (shift->tx || Carp::croak 'Transaction already destroyed')->req }
+sub res { (shift->tx || Carp::croak 'Transaction already destroyed')->res }
 
 sub send {
   my ($self, $msg, $cb) = @_;
-  my $tx = $self->tx || Carp::croak 'Connection already closed';
-  Carp::croak 'No WebSocket connection to send message to'
-    unless $tx->is_websocket;
+  my $tx = $self->tx || Carp::croak 'Transaction already destroyed';
+  Carp::croak 'No WebSocket connection to send message to' unless $tx->is_websocket;
   $tx->send($msg, $cb ? sub { shift; $self->$cb(@_) } : ());
   return $tx->established ? $self : $self->rendered(101);
 }
@@ -257,11 +207,10 @@ sub session {
   my $self = shift;
 
   my $stash = $self->stash;
-  $self->app->sessions->load($self)
-    unless exists $stash->{'mojo.active_session'};
+  $self->app->sessions->load($self) unless exists $stash->{'mojo.active_session'};
 
   # Hash
-  my $session = $stash->{'mojo.session'} ||= {};
+  my $session = $stash->{'mojo.session'} //= {};
   return $session unless @_;
 
   # Get
@@ -281,8 +230,8 @@ sub signed_cookie {
   return $self->every_signed_cookie($name)->[-1] unless defined $value;
 
   # Response cookie
-  my $checksum = Mojo::Util::hmac_sha1_sum($value, $self->app->secrets->[0]);
-  return $self->cookie($name, "$value--$checksum", $options);
+  my $sum = Digest::SHA::hmac_sha256_hex("$name=$value", $self->app->secrets->[0]);
+  return $self->cookie($name, "$value--$sum", $options);
 }
 
 sub stash { Mojo::Util::_stash(stash => @_) }
@@ -291,8 +240,8 @@ sub url_for {
   my ($self, $target) = (shift, shift // '');
 
   # Absolute URL
-  return $target if Scalar::Util::blessed $target && $target->isa('Mojo::URL');
-  return Mojo::URL->new($target) if $target =~ m!^(?:[^:/?#]+:|//|#)!;
+  return $target                 if Scalar::Util::blessed $target && $target->isa('Mojo::URL');
+  return Mojo::URL->new($target) if $target =~ $ABSOLUTE;
 
   # Base
   my $url  = Mojo::URL->new;
@@ -313,9 +262,8 @@ sub url_for {
   # Route
   else {
     my $generated = $self->match->path_for($target, @_);
-    $path->parse($generated->{path}) if $generated->{path};
-    $base->scheme($base->protocol eq 'https' ? 'wss' : 'ws')
-      if $generated->{websocket};
+    $path->parse($generated->{path})                         if $generated->{path};
+    $base->scheme($base->protocol eq 'https' ? 'wss' : 'ws') if $generated->{websocket};
   }
 
   # Make path absolute
@@ -326,20 +274,14 @@ sub url_for {
   return $url;
 }
 
-sub validation {
-  my $self = shift;
+sub url_for_asset {
+  my ($self, $asset) = @_;
+  return $self->url_for($asset =~ $ABSOLUTE ? $asset : $self->app->static->asset_path($asset));
+}
 
-  my $stash = $self->stash;
-  return $stash->{'mojo.validation'} if $stash->{'mojo.validation'};
-
-  my $req    = $self->req;
-  my $token  = $self->session->{csrf_token};
-  my $header = $req->headers->header('X-CSRF-Token');
-  my $hash   = $req->params->to_hash;
-  $hash->{csrf_token} //= $header if $token && $header;
-  $hash->{$_} = $req->every_upload($_) for map { $_->name } @{$req->uploads};
-  my $validation = $self->app->validator->validation->input($hash);
-  return $stash->{'mojo.validation'} = $validation->csrf_token($token);
+sub url_for_file {
+  my ($self, $file) = @_;
+  return $self->url_for($file =~ $ABSOLUTE ? $file : $self->app->static->file_path($file));
 }
 
 sub write {
@@ -350,8 +292,7 @@ sub write {
 
 sub write_chunk {
   my ($self, $chunk, $cb) = @_;
-  my $content = $self->res->content;
-  $content->write_chunk($chunk, $cb ? sub { shift; $self->$cb(@_) } : ());
+  $self->res->content->write_chunk($chunk, $cb ? sub { shift; $self->$cb(@_) } : ());
   return $self->rendered;
 }
 
@@ -367,11 +308,10 @@ Mojolicious::Controller - Controller base class
 
   # Controller
   package MyApp::Controller::Foo;
-  use Mojo::Base 'Mojolicious::Controller';
+  use Mojo::Base 'Mojolicious::Controller', -signatures;
 
   # Action
-  sub bar {
-    my $self = shift;
+  sub bar ($self) {
     my $name = $self->param('name');
     $self->res->headers->cache_control('max-age=1, no-cache');
     $self->render(json => {hello => $name});
@@ -379,22 +319,20 @@ Mojolicious::Controller - Controller base class
 
 =head1 DESCRIPTION
 
-L<Mojolicious::Controller> is the base class for your L<Mojolicious>
-controllers. It is also the default controller class unless you set
-L<Mojolicious/"controller_class">.
+L<Mojolicious::Controller> is the base class for your L<Mojolicious> controllers. It is also the default controller
+class unless you set L<Mojolicious/"controller_class">.
 
 =head1 ATTRIBUTES
 
-L<Mojolicious::Controller> inherits all attributes from L<Mojo::Base> and
-implements the following new ones.
+L<Mojolicious::Controller> implements the following attributes.
 
 =head2 app
 
   my $app = $c->app;
   $c      = $c->app(Mojolicious->new);
 
-A reference back to the application that dispatched to this controller, usually
-a L<Mojolicious> object.
+A reference back to the application that dispatched to this controller, usually a L<Mojolicious> object. Note that this
+attribute is weakened.
 
   # Use application logger
   $c->app->log->debug('Hello Mojo');
@@ -407,8 +345,7 @@ a L<Mojolicious> object.
   my $m = $c->match;
   $c    = $c->match(Mojolicious::Routes::Match->new);
 
-Router results for the current request, defaults to a
-L<Mojolicious::Routes::Match> object.
+Router results for the current request, defaults to a L<Mojolicious::Routes::Match> object.
 
   # Introspect
   my $name   = $c->match->endpoint->name;
@@ -420,11 +357,9 @@ L<Mojolicious::Routes::Match> object.
   my $tx = $c->tx;
   $c     = $c->tx(Mojo::Transaction::HTTP->new);
 
-The transaction that is currently being processed, usually a
-L<Mojo::Transaction::HTTP> or L<Mojo::Transaction::WebSocket> object. Note that
-this reference is usually weakened, so the object needs to be referenced
-elsewhere as well when you're performing non-blocking operations and the
-underlying connection might get closed early.
+The transaction that is currently being processed, usually a L<Mojo::Transaction::HTTP> or
+L<Mojo::Transaction::WebSocket> object. Note that this attribute is weakened. So the object needs to be referenced
+elsewhere as well when you're performing non-blocking operations and the underlying connection might get closed early.
 
   # Check peer information
   my $address = $c->tx->remote_address;
@@ -441,15 +376,13 @@ underlying connection might get closed early.
 
 =head1 METHODS
 
-L<Mojolicious::Controller> inherits all methods from L<Mojo::Base> and
-implements the following new ones.
+L<Mojolicious::Controller> inherits all methods from L<Mojo::Base> and implements the following new ones.
 
 =head2 continue
 
   $c->continue;
 
-Continue dispatch chain from an intermediate destination with
-L<Mojolicious::Routes/"continue">.
+Continue dispatch chain from an intermediate destination with L<Mojolicious::Routes/"continue">.
 
 =head2 cookie
 
@@ -457,9 +390,8 @@ L<Mojolicious::Routes/"continue">.
   $c        = $c->cookie(foo => 'bar');
   $c        = $c->cookie(foo => 'bar', {path => '/'});
 
-Access request cookie values and create new response cookies. If there are
-multiple values sharing the same name, and you want to access more than just
-the last one, you can use L</"every_cookie">.
+Access request cookie values and create new response cookies. If there are multiple values sharing the same name, and
+you want to access more than just the last one, you can use L</"every_cookie">.
 
   # Create response cookie with domain and expiration date
   $c->cookie(user => 'sri', {domain => 'example.com', expires => time + 60});
@@ -471,8 +403,7 @@ the last one, you can use L</"every_cookie">.
 
   my $values = $c->every_cookie('foo');
 
-Similar to L</"cookie">, but returns all request cookie values sharing the same
-name as an array reference.
+Similar to L</"cookie">, but returns all request cookie values sharing the same name as an array reference.
 
   $ Get first cookie value
   my $first = $c->every_cookie('foo')->[0];
@@ -481,8 +412,7 @@ name as an array reference.
 
   my $values = $c->every_param('foo');
 
-Similar to L</"param">, but returns all values sharing the same name as an
-array reference.
+Similar to L</"param">, but returns all values sharing the same name as an array reference.
 
   # Get first value
   my $first = $c->every_param('foo')->[0];
@@ -491,8 +421,8 @@ array reference.
 
   my $values = $c->every_signed_cookie('foo');
 
-Similar to L</"signed_cookie">, but returns all signed request cookie values
-sharing the same name as an array reference.
+Similar to L</"signed_cookie">, but returns all signed request cookie values sharing the same name as an array
+reference.
 
   # Get first signed cookie value
   my $first = $c->every_signed_cookie('foo')->[0];
@@ -504,29 +434,15 @@ sharing the same name as an array reference.
   $c = $c->finish(1003 => 'Cannot accept data!');
   $c = $c->finish('Bye!');
 
-Close WebSocket connection or long poll stream gracefully. This method will
-automatically respond to WebSocket handshake requests with a C<101> response
-status, to establish the WebSocket connection.
-
-=head2 flash
-
-  my $foo = $c->flash('foo');
-  $c      = $c->flash({foo => 'bar'});
-  $c      = $c->flash(foo => 'bar');
-
-Data storage persistent only for the next request, stored in the L</"session">.
-
-  # Show message after redirect
-  $c->flash(message => 'User created successfully!');
-  $c->redirect_to('show_user', id => 23);
+Close WebSocket connection or long poll stream gracefully. This method will automatically respond to WebSocket
+handshake requests with a C<101> response status, to establish the WebSocket connection.
 
 =head2 helpers
 
   my $helpers = $c->helpers;
 
-Return a proxy object containing the current controller object and on which
-helpers provided by L</"app"> can be called. This includes all helpers from
-L<Mojolicious::Plugin::DefaultHelpers> and L<Mojolicious::Plugin::TagHelpers>.
+Return a proxy object containing the current controller object and on which helpers provided by L</"app"> can be
+called. This includes all helpers from L<Mojolicious::Plugin::DefaultHelpers> and L<Mojolicious::Plugin::TagHelpers>.
 
   # Make sure to use the "title" helper and not the controller method
   $c->helpers->title('Welcome!');
@@ -538,32 +454,27 @@ L<Mojolicious::Plugin::DefaultHelpers> and L<Mojolicious::Plugin::TagHelpers>.
 
   my $cb = $c->on(finish => sub {...});
 
-Subscribe to events of L</"tx">, which is usually a L<Mojo::Transaction::HTTP>
-or L<Mojo::Transaction::WebSocket> object. This method will automatically
-respond to WebSocket handshake requests with a C<101> response status, to
+Subscribe to events of L</"tx">, which is usually a L<Mojo::Transaction::HTTP> or L<Mojo::Transaction::WebSocket>
+object. This method will automatically respond to WebSocket handshake requests with a C<101> response status, to
 establish the WebSocket connection.
 
   # Do something after the transaction has been finished
-  $c->on(finish => sub {
-    my $c = shift;
+  $c->on(finish => sub ($c) {
     $c->app->log->debug('All data has been sent');
   });
 
   # Receive WebSocket message
-  $c->on(message => sub {
-    my ($c, $msg) = @_;
+  $c->on(message => sub ($c, $msg) {
     $c->app->log->debug("Message: $msg");
   });
 
   # Receive JSON object via WebSocket message
-  $c->on(json => sub {
-    my ($c, $hash) = @_;
+  $c->on(json => sub ($c, $hash) {
     $c->app->log->debug("Test: $hash->{test}");
   });
 
   # Receive WebSocket "Binary" message
-  $c->on(binary => sub {
-    my ($c, $bytes) = @_;
+  $c->on(binary => sub ($c, $bytes) {
     my $len = length $bytes;
     $c->app->log->debug("Received $len bytes");
   });
@@ -575,14 +486,11 @@ establish the WebSocket connection.
   $c        = $c->param(foo => 'ba;r', 'baz');
   $c        = $c->param(foo => ['ba;r', 'baz']);
 
-Access route placeholder values that are not reserved stash values, file
-uploads as well as C<GET> and C<POST> parameters extracted from the query
-string and C<application/x-www-form-urlencoded> or C<multipart/form-data>
-message body, in that order. If there are multiple values sharing the same
-name, and you want to access more than just the last one, you can use
-L</"every_param">. Parts of the request body need to be loaded into memory to
-parse C<POST> parameters, so you have to make sure it is not excessively large.
-There's a 16MiB limit for requests by default.
+Access route placeholder values that are not reserved stash values, file uploads as well as C<GET> and C<POST>
+parameters extracted from the query string and C<application/x-www-form-urlencoded> or C<multipart/form-data> message
+body, in that order. If there are multiple values sharing the same name, and you want to access more than just the last
+one, you can use L</"every_param">. Parts of the request body need to be loaded into memory to parse C<POST>
+parameters, so you have to make sure it is not excessively large. There's a 16MiB limit for requests by default.
 
   # Get first value
   my $first = $c->every_param('foo')->[0];
@@ -601,24 +509,6 @@ For more control you can also access request information directly.
   # Only file uploads
   my $foo = $c->req->upload('foo');
 
-=head2 redirect_to
-
-  $c = $c->redirect_to('named', foo => 'bar');
-  $c = $c->redirect_to('named', {foo => 'bar'});
-  $c = $c->redirect_to('/index.html');
-  $c = $c->redirect_to('http://example.com/index.html');
-
-Prepare a C<302> (if the status code is not already C<3xx>) redirect response
-with C<Location> header, takes the same arguments as L</"url_for">.
-
-  # Moved Permanently
-  $c->res->code(301);
-  $c->redirect_to('some_route');
-
-  # Temporary Redirect
-  $c->res->code(307);
-  $c->redirect_to('some_route');
-
 =head2 render
 
   my $bool = $c->render;
@@ -631,11 +521,9 @@ with C<Location> header, takes the same arguments as L</"url_for">.
   my $bool = $c->render(handler => 'something');
   my $bool = $c->render('foo/index');
 
-Render content with L<Mojolicious/"renderer"> and emit hooks
-L<Mojolicious/"before_render"> as well as L<Mojolicious/"after_render">, or
-call L<Mojolicious::Plugin::DefaultHelpers/"reply-E<gt>not_found"> if no
-response could be generated, all additional key/value pairs get merged into the
-L</"stash">.
+Render content with L<Mojolicious/"renderer"> and emit hooks L<Mojolicious/"before_render"> as well as
+L<Mojolicious/"after_render">, or dies if no response could be generated. All additional key/value pairs get merged into
+the L</"stash">.
 
   # Render characters
   $c->render(text => 'I ♥ Mojolicious!');
@@ -644,7 +532,7 @@ L</"stash">.
   $c->stash(text => 'I ♥ Mojolicious!')->render;
 
   # Render binary data
-  use Mojo::JSON 'encode_json';
+  use Mojo::JSON qw(encode_json);
   $c->render(data => encode_json({test => 'I ♥ Mojolicious!'}));
 
   # Render JSON
@@ -669,14 +557,12 @@ L</"stash">.
 
   $c = $c->render_later;
 
-Disable automatic rendering to delay response generation, only necessary if
-automatic rendering would result in a response.
+Disable automatic rendering to delay response generation, only necessary if automatic rendering would result in a
+response.
 
   # Delayed rendering
   $c->render_later;
-  Mojo::IOLoop->timer(2 => sub {
-    $c->render(text => 'Delayed by 2 seconds!');
-  });
+  Mojo::IOLoop->timer(2 => sub { $c->render(text => 'Delayed by 2 seconds!') });
 
 =head2 render_maybe
 
@@ -684,9 +570,9 @@ automatic rendering would result in a response.
   my $bool = $c->render_maybe(foo => 'bar', baz => 23);
   my $bool = $c->render_maybe('foo/index', format => 'html');
 
-Try to render content, but do not call
-L<Mojolicious::Plugin::DefaultHelpers/"reply-E<gt>not_found"> if no response
-could be generated, takes the same arguments as L</"render">.
+Try to render content, but do not call L<Mojolicious::Plugin::DefaultHelpers/"reply-E<gt>not_found"> if no response
+could be generated, all arguments get localized automatically and are only available during this render operation,
+takes the same arguments as L</"render">.
 
   # Render template "index_local" only if it exists
   $c->render_maybe('index_local') or $c->render('index');
@@ -695,10 +581,8 @@ could be generated, takes the same arguments as L</"render">.
 
   my $output = $c->render_to_string('foo/index', format => 'pdf');
 
-Try to render content and return it wrapped in a L<Mojo::ByteStream> object or
-return C<undef>, all arguments get localized automatically and are only
-available during this render operation, takes the same arguments as
-L</"render">.
+Try to render content and return it wrapped in a L<Mojo::ByteStream> object or return C<undef>, all arguments get
+localized automatically and are only available during this render operation, takes the same arguments as L</"render">.
 
   # Render inline template
   my $two = $c->render_to_string(inline => '<%= 1 + 1 %>');
@@ -708,8 +592,7 @@ L</"render">.
   $c = $c->rendered;
   $c = $c->rendered(302);
 
-Finalize response and emit hook L<Mojolicious/"after_dispatch">, defaults to
-using a C<200> response code.
+Finalize response and emit hook L<Mojolicious/"after_dispatch">, defaults to using a C<200> response code.
 
   # Custom response
   $c->res->headers->content_type('text/plain');
@@ -726,6 +609,7 @@ Get L<Mojo::Message::Request> object from L</"tx">.
   my $req = $c->tx->req;
 
   # Extract request information
+  my $id     = $c->req->request_id;
   my $method = $c->req->method;
   my $url    = $c->req->url->to_abs;
   my $info   = $c->req->url->to_abs->userinfo;
@@ -760,30 +644,6 @@ Get L<Mojo::Message::Response> object from L</"tx">.
   $c->res->headers->cache_control('public, max-age=300');
   $c->res->headers->append(Vary => 'Accept-Encoding');
 
-=head2 respond_to
-
-  $c = $c->respond_to(
-    json => {json => {message => 'Welcome!'}},
-    html => {template => 'welcome'},
-    any  => sub {...}
-  );
-
-Automatically select best possible representation for resource from C<format>
-stash value, C<format> C<GET>/C<POST> parameter or C<Accept> request header,
-defaults to L<Mojolicious::Renderer/"default_format"> or rendering an empty
-C<204> response. Each representation can be handled with a callback or a hash
-reference containing arguments to be passed to L</"render">.
-
-  # Everything else than "json" and "xml" gets a 204 response
-  $c->respond_to(
-    json => sub { $c->render(json => {just => 'works'}) },
-    xml  => {text => '<just>works</just>'},
-    any  => {data => '', status => 204}
-  );
-
-For more advanced negotiation logic you can also use the helper
-L<Mojolicious::Plugin::DefaultHelpers/"accepts">.
-
 =head2 send
 
   $c = $c->send({binary => $bytes});
@@ -791,11 +651,10 @@ L<Mojolicious::Plugin::DefaultHelpers/"accepts">.
   $c = $c->send({json   => {test => [1, 2, 3]}});
   $c = $c->send([$fin, $rsv1, $rsv2, $rsv3, $op, $payload]);
   $c = $c->send($chars);
-  $c = $c->send($chars => sub {...});
+  $c = $c->send($chars => sub ($c) {...});
 
-Send message or frame non-blocking via WebSocket, the optional drain callback
-will be executed once all data has been written. This method will automatically
-respond to WebSocket handshake requests with a C<101> response status, to
+Send message or frame non-blocking via WebSocket, the optional drain callback will be executed once all data has been
+written. This method will automatically respond to WebSocket handshake requests with a C<101> response status, to
 establish the WebSocket connection.
 
   # Send "Text" message
@@ -805,22 +664,18 @@ establish the WebSocket connection.
   $c->send({json => {test => 'I ♥ Mojolicious!'}});
 
   # Send JSON object as "Binary" message
-  use Mojo::JSON 'encode_json';
+  use Mojo::JSON qw(encode_json);
   $c->send({binary => encode_json({test => 'I ♥ Mojolicious!'})});
 
   # Send "Ping" frame
-  use Mojo::WebSocket 'WS_PING';
+  use Mojo::WebSocket qw(WS_PING);
   $c->send([1, 0, 0, 0, WS_PING, 'Hello World!']);
 
   # Make sure the first message has been written before continuing
-  $c->send('First message!' => sub {
-    my $c = shift;
-    $c->send('Second message!');
-  });
+  $c->send('First message!' => sub ($c) { $c->send('Second message!') });
 
-For mostly idle WebSockets you might also want to increase the inactivity
-timeout with L<Mojolicious::Plugin::DefaultHelpers/"inactivity_timeout">, which
-usually defaults to C<15> seconds.
+For mostly idle WebSockets you might also want to increase the inactivity timeout with
+L<Mojolicious::Plugin::DefaultHelpers/"inactivity_timeout">, which usually defaults to C<30> seconds.
 
   # Increase inactivity timeout for connection to 300 seconds
   $c->inactivity_timeout(300);
@@ -832,9 +687,8 @@ usually defaults to C<15> seconds.
   $c          = $c->session({foo => 'bar'});
   $c          = $c->session(foo => 'bar');
 
-Persistent data storage for the next few requests, all session data gets
-serialized with L<Mojo::JSON> and stored Base64 encoded in HMAC-SHA1 signed
-cookies, to prevent tampering. Note that cookies usually have a C<4096> byte
+Persistent data storage for the next few requests, all session data gets serialized with L<Mojo::JSON> and stored
+Base64 encoded in HMAC-SHA256 signed cookies, to prevent tampering. Note that cookies usually have a C<4096> byte
 (4KiB) limit, depending on browser.
 
   # Manipulate session
@@ -857,11 +711,10 @@ cookies, to prevent tampering. Note that cookies usually have a C<4096> byte
   $c        = $c->signed_cookie(foo => 'bar');
   $c        = $c->signed_cookie(foo => 'bar', {path => '/'});
 
-Access signed request cookie values and create new signed response cookies. If
-there are multiple values sharing the same name, and you want to access more
-than just the last one, you can use L</"every_signed_cookie">. Cookies are
-cryptographically signed with HMAC-SHA1, to prevent tampering, and the ones
-failing signature verification will be automatically discarded.
+Access signed request cookie values and create new signed response cookies. If there are multiple values sharing the
+same name, and you want to access more than just the last one, you can use L</"every_signed_cookie">. Cookies are
+cryptographically signed with HMAC-SHA256, to prevent tampering, and the ones failing signature verification will be
+automatically discarded.
 
 =head2 stash
 
@@ -870,13 +723,11 @@ failing signature verification will be automatically discarded.
   $c       = $c->stash({foo => 'bar', baz => 23});
   $c       = $c->stash(foo => 'bar', baz => 23);
 
-Non-persistent data storage and exchange for the current request, application
-wide default values can be set with L<Mojolicious/"defaults">. Some stash
-values have a special meaning and are reserved, the full list is currently
-C<action>, C<app>, C<cb>, C<controller>, C<data>, C<extends>, C<format>,
-C<handler>, C<inline>, C<json>, C<layout>, C<namespace>, C<path>, C<status>,
-C<template>, C<text> and C<variant>. Note that all stash values with a
-C<mojo.*> prefix are reserved for internal use.
+Non-persistent data storage and exchange for the current request, application wide default values can be set with
+L<Mojolicious/"defaults">. Some stash values have a special meaning and are reserved, the full list is currently
+C<action>, C<app>, C<cb>, C<controller>, C<data>, C<extends>, C<format>, C<handler>, C<inline>, C<json>, C<layout>,
+C<namespace>, C<path>, C<status>, C<template>, C<text> and C<variant>. Note that all stash values with a C<mojo.*>
+prefix are reserved for internal use.
 
   # Remove value
   my $foo = delete $c->stash->{foo};
@@ -923,63 +774,46 @@ Generate a portable L<Mojo::URL> object with base for a path, URL or route.
   # "/myapp/index.html?foo=bar" if application is deployed under "/myapp"
   $c->url_for('/index.html')->query(foo => 'bar');
 
-You can also use the helper L<Mojolicious::Plugin::DefaultHelpers/"url_with">
-to inherit query parameters from the current request.
+You can also use the helper L<Mojolicious::Plugin::DefaultHelpers/"url_with"> to inherit query parameters from the
+current request.
 
   # "/list?q=mojo&page=2" if current request was for "/list?q=mojo&page=1"
-  $c->url_with->query([page => 2]);
+  $c->url_with->query({page => 2});
 
-=head2 validation
 
-  my $validation = $c->validation;
+=head2 url_for_asset
 
-Get L<Mojolicious::Validator::Validation> object for current request to
-validate file uploads as well as C<GET> and C<POST> parameters extracted from
-the query string and C<application/x-www-form-urlencoded> or
-C<multipart/form-data> message body. Parts of the request body need to be loaded
-into memory to parse C<POST> parameters, so you have to make sure it is not
-excessively large. There's a 16MiB limit for requests by default.
+  my $url = $c->url_for_asset('/app.js');
 
-  # Validate GET/POST parameter
-  my $validation = $c->validation;
-  $validation->required('title', 'trim')->size(3, 50);
-  my $title = $validation->param('title');
+Generate a portable L<Mojo::URL> object with base for a static asset.
 
-  # Validate file upload
-  my $validation = $c->validation;
-  $validation->required('tarball')->upload->size(1, 1048576);
-  my $tarball = $validation->param('tarball');
+=head2 url_for_file
+
+  my $url = $c->url_for_file('/index.html');
+
+Generate a portable L<Mojo::URL> object with base for a static file.
 
 =head2 write
 
   $c = $c->write;
   $c = $c->write('');
   $c = $c->write($bytes);
-  $c = $c->write($bytes => sub {...});
+  $c = $c->write($bytes => sub ($c) {...});
 
-Write dynamic content non-blocking, the optional drain callback will be executed
-once all data has been written. Calling this method without a chunk of data
-will finalize the response headers and allow for dynamic content to be written
-later.
+Write dynamic content non-blocking, the optional drain callback will be executed once all data has been written.
+Calling this method without a chunk of data will finalize the response headers and allow for dynamic content to be
+written later.
 
   # Keep connection alive (with Content-Length header)
   $c->res->headers->content_length(6);
-  $c->write('Hel' => sub {
-    my $c = shift;
-    $c->write('lo!');
-  });
+  $c->write('Hel' => sub ($c) { $c->write('lo!') });
 
   # Close connection when finished (without Content-Length header)
-  $c->write('Hel' => sub {
-    my $c = shift;
-    $c->write('lo!' => sub {
-      my $c = shift;
-      $c->finish;
-    });
+  $c->write('Hel' => sub ($c) {
+    $c->write('lo!' => sub ($c) { $c->finish });
   });
 
-You can call L</"finish"> or write an empty chunk of data at any time to end
-the stream.
+You can call L</"finish"> or write an empty chunk of data at any time to end the stream.
 
   HTTP/1.1 200 OK
   Date: Sat, 13 Sep 2014 16:48:29 GMT
@@ -995,9 +829,8 @@ the stream.
 
   Hello!
 
-For Comet (long polling) you might also want to increase the inactivity timeout
-with L<Mojolicious::Plugin::DefaultHelpers/"inactivity_timeout">, which usually
-defaults to C<15> seconds.
+For Comet (long polling) you might also want to increase the inactivity timeout with
+L<Mojolicious::Plugin::DefaultHelpers/"inactivity_timeout">, which usually defaults to C<30> seconds.
 
   # Increase inactivity timeout for connection to 300 seconds
   $c->inactivity_timeout(300);
@@ -1007,24 +840,18 @@ defaults to C<15> seconds.
   $c = $c->write_chunk;
   $c = $c->write_chunk('');
   $c = $c->write_chunk($bytes);
-  $c = $c->write_chunk($bytes => sub {...});
+  $c = $c->write_chunk($bytes => sub ($c) {...});
 
-Write dynamic content non-blocking with chunked transfer encoding, the optional
-drain callback will be executed once all data has been written. Calling this
-method without a chunk of data will finalize the response headers and allow for
+Write dynamic content non-blocking with chunked transfer encoding, the optional drain callback will be executed once
+all data has been written. Calling this method without a chunk of data will finalize the response headers and allow for
 dynamic content to be written later.
 
   # Make sure previous chunk has been written before continuing
-  $c->write_chunk('H' => sub {
-    my $c = shift;
-    $c->write_chunk('ell' => sub {
-      my $c = shift;
-      $c->finish('o!');
-    });
+  $c->write_chunk('H' => sub ($c) {
+    $c->write_chunk('ell' => sub ($c) { $c->finish('o!') });
   });
 
-You can call L</"finish"> or write an empty chunk of data at any time to end
-the stream.
+You can call L</"finish"> or write an empty chunk of data at any time to end the stream.
 
   HTTP/1.1 200 OK
   Date: Sat, 13 Sep 2014 16:48:29 GMT
@@ -1039,11 +866,10 @@ the stream.
   o!
   0
 
-=head1 AUTOLOAD
+=head1 HELPERS
 
-In addition to the L</"ATTRIBUTES"> and L</"METHODS"> above you can also call
-helpers provided by L</"app"> on L<Mojolicious::Controller> objects. This
-includes all helpers from L<Mojolicious::Plugin::DefaultHelpers> and
+In addition to the L</"ATTRIBUTES"> and L</"METHODS"> above you can also call helpers provided by L</"app"> on
+L<Mojolicious::Controller> objects. This includes all helpers from L<Mojolicious::Plugin::DefaultHelpers> and
 L<Mojolicious::Plugin::TagHelpers>.
 
   # Call helpers
@@ -1055,6 +881,6 @@ L<Mojolicious::Plugin::TagHelpers>.
 
 =head1 SEE ALSO
 
-L<Mojolicious>, L<Mojolicious::Guides>, L<http://mojolicious.org>.
+L<Mojolicious>, L<Mojolicious::Guides>, L<https://mojolicious.org>.
 
 =cut
